/*
 * ******************************************************************************************
 *
 * File		: apb_pit.h
 * Author	: GOWIN
 * Function	: Definition of APB PIT
 *
 * ******************************************************************************************
 */

#ifndef __APB_PIT_H__
#define __APB_PIT_H__


#ifdef __cplusplus
extern "C" {
#endif


// Includes ---------------------------------------------------------------------------------
#include <stddef.h>


/*****************************************************************************
 * Device Specific Peripheral Registers structures
 ****************************************************************************/
#define __RO          volatile const  /* 'read  only' permissions     */
#define __WO          volatile        /* 'write only' permissions     */
#define __RW          volatile        /* 'read/write' permissions     */


/*****************************************************************************
 * APB PIT (Programmable Interval Timer)
 * APB PWM (Pulse Width Modulator)
 ****************************************************************************/
/*
 * APB PIT or APB PWM
 *
 * 32-bit Timer 0
 * 16-bit Timer 0, 16-bit Timer 1
 *  8-bit Timer 0,  8-bit Timer 1, 8-bit Timer 2, 8-bit Timer3
 *                                                             16-bit PWM
 * 16-bit Timer 0,                                              8-bit PWM
 *  8-bit Timer 0,  8-bit Timer 1,                              8-bit PWM
 */
typedef struct
{
	__RW unsigned int CTRL;                 /* APB PIT Channel Control Register */
	__RW unsigned int RELOAD;               /* APB PIT Channel Reload Register  */
	__RW unsigned int COUNTER;              /* APB PIT Channel Counter Register */
	     unsigned int RESERVED[1];			/* Reserved                         */
} APB_PIT_CHANNEL_REG;

typedef struct
{
	__RO unsigned int     IDREV;                /* 0x00 ID and Revision Register  */
	     unsigned int     RESERVED[3];          /* 0x04~0x0C Reserved             */
	__RO unsigned int     CFG;                  /* 0x10 Configuration Register    */
	__RW unsigned int     INTEN;                /* 0x14 Interrupt Enable Register */
	__RW unsigned int     INTST;                /* 0x18 Interrupt Status Register */
	__RW unsigned int     CHNEN;                /* 0x1C Channel Enable Register   */
	APB_PIT_CHANNEL_REG   CHANNEL[4];           /* 0x20~0x50 Channel #n Registers */
} APB_PIT_RegDef;


/* General return codes */
#define APB_PIT_DRIVER_OK                 0 // Operation succeeded
#define APB_PIT_DRIVER_ERROR             -1 // Unspecified error
#define APB_PIT_DRIVER_ERROR_BUSY        -2 // Driver is busy
#define APB_PIT_DRIVER_ERROR_TIMEOUT     -3 // Timeout occurred
#define APB_PIT_DRIVER_ERROR_UNSUPPORTED -4 // Operation not supported
#define APB_PIT_DRIVER_ERROR_PARAMETER   -5 // Parameter error
#define APB_PIT_DRIVER_ERROR_SPECIFIC    -6 // Start of driver specific errors


#ifdef __cplusplus
}
#endif


#endif	// __APB_PIT_H__
