/*
 * ******************************************************************************************
 *
 * File		: apb_pit_driver.h
 * Author	: GOWIN
 * Function	: Driver definition of APB PIT
 *
 * ******************************************************************************************
 */

#ifndef __APB_PIT_DRIVER_H__
#define __APB_PIT_DRIVER_H__


#ifdef __cplusplus
extern "C" {
#endif


// Definitions ------------------------------------------------------------------------------

// APB PIT control command
#define APB_PIT_TIMER_START					(1UL << 0)		// Start timer
#define APB_PIT_TIMER_STOP					(1UL << 1)		// Stop timer
#define APB_PIT_TIMER_INTR_ENABLE			(1UL << 2)		// Enable timer interrupt
#define APB_PIT_TIMER_INTR_DISABLE			(1UL << 3)		// Disable timer interrupt
#define APB_PIT_TIMER_INTR_CLEAR			(1UL << 4)		// Clear timer interrupt

#define	APB_PIT_TIMER_SEC_TICK				(1UL << 0)		// Second tick
#define APB_PIT_TIMER_MSEC_TICK				(1UL << 1)		// Microsecond tick

// Number of APB PIT channels
#define APB_PIT_NUMBER_OF_CHANNELS          (4)

#define APB_PIT_CHNCTRL_CLK_EXTERNAL        (0 << 3)
#define APB_PIT_CHNCTRL_CLK_PCLK            (1 << 3)
#define APB_PIT_CHNCTRL_MODEMASK            0x07
#define APB_PIT_CHNCTRL_TMR_32BIT           1
#define APB_PIT_CHNCTRL_TMR_16BIT           2
#define APB_PIT_CHNCTRL_TMR_8BIT            3
#define APB_PIT_CHNCTRL_PWM                 4
#define APB_PIT_CHNCTRL_MIXED_16BIT         6
#define APB_PIT_CHNCTRL_MIXED_8BIT          7


//**********************************************
// Functions
//**********************************************

// Initializes APB PIT
int apb_pit_init(void);

// Read time
unsigned int apb_pit_read(unsigned int tmr);

// Control
int apb_pit_control(unsigned int mode, unsigned int tmr);

// Set period
int apb_pit_set_period(unsigned int tmr, unsigned int period);

// Get status
unsigned int apb_pit_get_status(unsigned int tmr);

// Get (m)s tick
unsigned int apb_pit_get_tick(unsigned int mode, unsigned int sec);


#ifdef __cplusplus
}
#endif


#endif	/* __APB_PIT_DRIVER_H__ */
