/*
 * ******************************************************************************************
 *
 * File		: main.c
 * Author	: GOWIN
 * Function	: APB PIT and PWM demo
 *
 * ******************************************************************************************
 */

// Includes ---------------------------------------------------------------------------------
#include "apb_pit_driver.h"
#include "apb_pwm_driver.h"
#include "apb_pit_dev.h"
#include "uart.h"
#include <stdio.h>


// Definitions ------------------------------------------------------------------------------


//*******************************
// APB PWM demo
//*******************************

#define APB_PWM_OUPUT_CHANNEL	0	 // APB PWM channel 0

#define NOTE_DO                 262  //523
#define NOTE_RE                 294  //587
#define NOTE_MI                 330  //659
#define NOTE_FA                 349  //698
#define NOTE_SO                 392  //784
#define NOTE_LA                 440  //880
#define NOTE_SE                 494  //988

static const unsigned int tones[] =
{
    NOTE_DO, NOTE_RE, NOTE_MI, NOTE_FA, NOTE_SO, NOTE_LA, NOTE_SE
};

// APB PWM demo
int apb_pwm_demo(void)
{
    char apb_pwm_num = APB_PWM_OUPUT_CHANNEL;
    char note;

    /* Initializes UART */
    uart_init(38400);	// Baud rate is 38400

    printf("\r\nIt's a APB PWM demo.\r\n\r\n");

    /* Initializes the APB PWM driver */
    apb_pwm_initialize(NULL);

    /* Power up the APB PWM peripheral */
    apb_pwm_power_control(APB_PWM_POWER_FULL);

    /* Active the APB PWM output channel */
    apb_pwm_control(APB_PWM_ACTIVE_CONFIGURE |
    		        APB_PWM_PARK_LOW |
					APB_PWM_CLKSRC_EXTERNAL, apb_pwm_num);

    printf("Let's play piano...\r\n");
    printf("[1 2 3 4 5 6 7] ==> Play [Do Re Mi Fa So La Si]\r\n");
    printf("[Space]         ==> Mute\r\n");
    printf("[Enter]         ==> Quit\r\n");

    while(1)
    {
        /* Input a character from UART */
    	note = uart_getc();

        if (note == 13)
        {
        	/* If input Enter, quit */
        	printf("Input a [Enter]\r\n");

        	apb_pwm_power_control(APB_PWM_POWER_OFF);
            printf("\r\nQUIT!");

            break;
        }
        else if (note > 48 && note < 56)
        {
        	/* If input 1 ~ 7, play note */
        	printf("Input a [%d]\r\n", note);

        	apb_pwm_set_freq(apb_pwm_num, tones[note - 49]);
        	apb_pwm_output(apb_pwm_num, 255/2);  /* Output 50% duty cycle */
        }
        else if (note == 32)
        {
            /* If input Space, mute */
        	printf("Input a [Space]\r\n");

        	apb_pwm_output(apb_pwm_num, 0);      /* Output 0% duty cycle to disable */
        }
    }

    return 0;
}


//*******************************
// APB PIT demo
//*******************************

#define APB_PIT_TIMER_PERIOD	(4 * OSCFREQ)	// 4 secs

#define APB_PIT_TIMER0	0	// APB PIT 0
#define APB_PIT_TIMER1	1	// APB PIT 1
#define APB_PIT_TIMER2	2	// APB PIT 2
#define APB_PIT_TIMER3	3	// APB PIT 3

// Setup APB PIT
static void setup_apb_pit_timer(unsigned int period)
{
	// Initializes APB PIT
	apb_pit_init();

	// Set period
	apb_pit_set_period(APB_PIT_TIMER0, period);

	// Enable interrupt
	apb_pit_control(APB_PIT_TIMER_INTR_ENABLE, APB_PIT_TIMER0);

	// Set the interrupt level 1 for the APB PIT
	HAL_INTERRUPT_SET_LEVEL(IRQ_APB_PIT_SOURCE, 1);

	/* Enable PLIC interrupt APB PIT source */
	HAL_INTERRUPT_ENABLE(IRQ_APB_PIT_SOURCE);

	/* Start APB PIT */
	apb_pit_control(APB_PIT_TIMER_START, APB_PIT_TIMER0);

	printf("Setup APB PIT.\r\n");
}

// APB PIT interrupt handler
void apb_pit_irq_handler(void)
{
	/* Clear APB PIT interrupt status */
	apb_pit_control(APB_PIT_TIMER_INTR_CLEAR, APB_PIT_TIMER0);

	printf("It's in APB PIT interrupt.\r\n");
}

// APB PIT demo
int apb_pit_demo(void)
{
	// Initializes UART
	uart_init(38400);	// Baud rate is 38400

	printf("\r\nIt's a APB PIT demo.\r\n\r\n");

	// Disable Machine External interrupts
	HAL_MEIP_DISABLE();

	// Setup APB PIT
	setup_apb_pit_timer(APB_PIT_TIMER_PERIOD);

	// Enable Machine External interrupts
	HAL_MEIP_ENABLE();

	// Enable interrupts in general.
	HAL_MIE_ENABLE();

	return 0;
}


/* Application entry function */
int main(void)
{
	apb_pwm_demo();
	//apb_pit_demo();

    return 0;
}
